#!/perl/bin/perl
#
# Spong client monitoring script.  This runs various tests locally on a machine
# and sends those results to the Spong server machine where the information
# is summarized and displayed.  This script currently checks the following:
#
#    * disk  (check to make sure that disks are not filling up)
#    * cpu   (check to make sure the load is reasonable)
#    * jobs  (check to make sure certain key processes are running)
#	 * services (check to make sure certain key services are
# 				 running)
#
# History:
# (1) Ported bb-local.sh script to perl. (Ed Hill Feb 26, 1997)
# (2) Ported the spong client to Windows NT (Joanna Ziembicki, July 10, 2000)

use lib 'C:/SpongClient/lib';

use Sys::Hostname;
use Socket;
use POSIX;
use Getopt::Long;

use Spong::Status qw(status);

use Win32;
use Win32::Daemon;
use Win32::Service;

srand( time() ^ ($$ + ($$ << 15 )) );

$VERSION = "1.0";
$SERVICENAME ="Spong-NT-Client";
$SPONGPATH = "c:\\SpongClient";

$me = "$SPONGPATH\\spong-nt-client.pl";
$conf_file = "$SPONGPATH\\etc\\spong.cf";

my %Options;  # a global variable (eww..) which hold the options
			  # passed to the program

$Result = Getopt::Long::GetOptions (\%Options, 
					  qw(
						debug|d
					  	install|i
						remove|r
						help|?|h
						restart
						stop
						nosleep
						start|s
                        ) 
					  );

#---------------------------------
# Handle command-line parameters
#---------------------------------

# Install or remove the Win32 service

if ( $Options{"install"} ) {
	&install();
	exit()
} elsif ( $Options{"remove"} ) {
	&remove();
	exit();
} elsif ( $Options{"start"} ) {
    &start();
    exit();
} elsif ( $Options{"restart"} ) {
    &restart();
    exit();
} elsif ( $Options{"stop"} ) {
    &stop();
    exit();
} elsif ( $Options{"help"} ) {
	&displaySyntax();
    exit();
} elsif ( $Options{"debug"} ) {
	&debug("Running in debug mode\n");
}

#-------------------------
# The script itself
#-------------------------

# Pretty simple program, check the stuff that we are looking at, report it to
# the server, sleep, and do it all again...
# Load configuration files and modules

$CHECKS = "";
&load_config_files(); # Loads the user specified configuration information
%CHECKFUNCS = (); 
&load_checks();

#--------------------------------------------------------
# Set up aliases for the Win32 service state identifiers
#--------------------------------------------------------

%STATE_ALIAS = ( '1' => 'SERVICE_STOPPED',
				 '2' => 'SERVICE_START_PENDING',
				 '3' => 'SERVICE_STOP_PENDING',
				 '4' => 'SERVICE_RUNNING',
				 '5' => 'SERVICE_CONTINUE_PENDING',
				 '6' => 'SERVICE_PAUSE_PENDING',
				 '7' => 'SERVICE_PAUSED',
				);


# If the debug option is turned on, run inside a command prompt window
# Otherwise, start as a Win32 service

if ( $Options{"debug"} ) {

	while (1) {
		my $check;
		foreach $check (split / /, $CHECKS) {
			eval {&{$CHECKFUNCS{$check}}();};
			if ($@) { &error( "Error running check $check: $@" ) };
		}

		# If we are supposed to stay alive, then sleep about $SPONGSLEEP 
		# seconds; we add a little randomness so that things don't get in 
		# sync and pound the spong-server.  
		# Otherwise, just exit.

		if( $Options{"nosleep"} ) {
			last;
		} else {
			my $sleep = int( $SPONGSLEEP - (0.05 * $SPONGSLEEP)  
					+ rand (0.1 * $SPONGSLEEP) );
			&debug( "sleeping for $sleep seconds" );
			sleep $sleep;
		}
	}
	exit(0);

} else {
	# Run spong as a Win32 service
	# uses Win32::Daemon module and associated script samples by Dave Roth
	# see www.roth.net for details
	
	# Start the service
    if ( Win32::Daemon::StartService() ) {
    	&log( "Service Started" );	
	} else {    
		&error ("Failed to start this script as a Win32 service: " . &getError . "\n");
		
	}
	
  	# Handle changes in service status 

	my $error_state_count = 0;
	
	while ( SERVICE_STOPPED != ($State = Win32::Daemon::State()) ) {
		if ( SERVICE_START_PENDING == $State )  	
		{
			# Initialization code
			&log ("Starting Spong client");
			
			Win32::Daemon::State( SERVICE_RUNNING );
		}
		elsif ( SERVICE_PAUSE_PENDING == $State) 
		{
			# Pausing...
			&log ("Pausing Spong client");
			Win32::Daemon::State( SERVICE_PAUSED );
			next;
		}
		elsif ( SERVICE_CONTINUE_PENDING == $State )
		{
			# Resuming...
			&log ("Resuming Spong client");
			Win32::Daemon::State( SERVICE_RUNNING );
			next;
		}
		elsif ( SERVICE_STOP_PENDING == $State ) 
		{
			# Stopping...
			&log ("Stopping Spong client");
			Win32::Daemon::State (SERVICE_STOPPED);
		}
		elsif ( SERVICE_RUNNING == $State )
		{
			# The service is running as normal..
			my ($check);

			foreach $check (split / /, $CHECKS) {
				eval { &{$CHECKFUNCS{$check}}(); };
				if ($@) { &log( "Error running check $check: $@") };
			}
			
            my $sleep = int($SPONGSLEEP - (.05 * $SPONGSLEEP) + 
            rand(.1 * $SPONGSLEEP));
			
			my $naptimes = int ($sleep/2);
			for $i (1..$naptimes) { 	
				sleep 2;  # Take little naps and keep checking for service state
				$State = Win32::Daemon::State();
				if ($State != SERVICE_RUNNING) {
					last; # Break out of the for-loop to handle state change
				} # end if 
			} # end for
		}
		else
        {
			# If service reports a strange/invalid state, log it.
			# Only log it up to 10 times to prevent the log file
			# from getting huge if the service crashes
			if ( ($error_state_count <= 10) 
				&& ($State ne SERVICE_PAUSED) ) {
				&log ("Service spong-nt-client reports state: $State.");
				$error_state_count ++;
			}
			
			sleep (2);
			
		}
	}
	
	# Stopping the service
	 &log("Stopping service.");
     Win32::Daemon::StopService();	
}


# ============================================ 
# Procedures for loading configuration and
# local checks
# ============================================ 

sub load_config_files {
	# Load our configuration variables, including anything 
	# specific to the host that we are running on.

   	$result = eval {require $conf_file};   
	
	if ($@) {
		if ($Options{"debug"}) {
			die("Error: $@");
		} else {	
			&stop;
		}
	}
	
	$HOST      =~ tr/A-Z/a-z/; # convert the hostname to lowercase
    &debug( "configuration file(s) loaded" );
}


sub load_checks {
    # Load client checking functions as defined the $HOSTS checks field
    # Default to old checks if 'checks' is not present (cpu disk services 
    # jobs [local])

#    if (! defined $CHECKS || ! $CHECKS ) {
#        $CHECKS = 'cpu disk services processes' ;
#        if (defined &check_local) { $CHECKS .= ' local'; }
#    };

    foreach $check (split / /,$CHECKS) {
        &debug( "Loading client check $check" );
        eval ('require "c:/SpongClient/lib/Spong/plugins/check_$check"');
        if ( $@ ) { &error( "Could not load $check check plugin" ); }
    }
}


# ========================================================== 
# Procedures for installing, removing and configuring Spong
# as a Win32 service
# ========================================================== 

sub getServiceConfig {

	my $ScriptPath = join ("", Win32::GetFullPathName( $0 ));
	my %Hash = (
		name => $SERVICENAME,
		display => 'Spong Network Monitor - NT Client',
		path => $^X,
		user => "",
		pwd => "",
		parameters => "$ScriptPath",
	);

	return (\%Hash );
}

sub install {
# Installs the spong-nt-client service
	my $ServiceConfig = &getServiceConfig();

	if ( Win32::Daemon::CreateService( $ServiceConfig ) ) {
		print "The $ServiceConfig->{display} was successfully installed\n";
	} else {
		print "Failed to add the $ServiceConfig->{display} service \n";
		print "Error: " . &getError() ."\n";
	}
}

sub remove {
# Uninstalls the spong-nt-client service from the registry
	my $ServiceConfig = &getServiceConfig();

	if ( Win32::Daemon::DeleteService( $ServiceConfig->{"name"} ) ) {
		print "The $ServiceConfig->{display} was successfully removed\n";
	} else {
		print "Failed to remove the $ServiceConfig->{display} service \n";
		print "Error: " . &getError() ."\n";
	}
}

sub start {
# Starts the spong-nt-client service
	my $ServiceConfig = &getServiceConfig();
    if (Win32::Service::StartService('', $SERVICENAME)) {
        print "The $ServiceConfig->{display} was successfully started.\n";
    } else {
        print "Failed to start the $ServiceConfig->{display} ($SERVICENAME) as an NT service: $!\n";
    }
}

sub stop {
# Stops the spong-nt-client service
	my $ServiceConfig = &getServiceConfig();
    if (Win32::Service::StopService('', $SERVICENAME)) {
        print "The $ServiceConfig->{display} was successfully stopped.\n";
    } else {
        print "Failed to stop the $ServiceConfig->{display} ($SERVICENAME) service: $!\n";
    }
}

sub restart {
# Restarts the spong-nt-client service
    my $ServiceConfig = &getServiceConfig();
    my @errors;
	
    if (Win32::Service::StopService('', $SERVICENAME)) {
        sleep 10;
        if ( Win32::Service::StartService ('', $SERVICENAME) ) {
            print "$ServiceConfig->{display} was successfully restarted.\n";
        	return 1;
		} else {
            push (@errors, "$!\n");
        }
     }   
	# Function has not yet returned, so there must be errors...
	push (@errors, "$!\n");
    print ("Failed to restart the $ServiceConfig->{display}: @errors\nIf service is not already running, please use $SCRIPTNAME --start");
	return 0;

}
    
   

# =================================================== 
# Procedures for displaying errors and help messages
# =================================================== 

sub getError {
# Returns the last error reported by Win32::Daemon
	return ( Win32::FormatMessage( Win32::Daemon::GetLastError() ) );
}

sub debug { 
# prints a message to standard output if we are in debug mode
	if ( $Options{"debug"} ) {
		print scalar localtime, " ", $_[0], "\n"; 
	}
}
	
sub log {
# Writes a message to the log file
	my $msg = shift;
	open (LOG, ">>$LOGFILE") 
		|| &debug ("Cannot open logfile"); 
		print LOG scalar localtime, " $msg \n";
	close (LOG);
}

sub error { 
    my $msg = shift;
	warn scalar localtime(), " Error: $msg \n"; 
	# Also print error to logfile if one is open and we are not
	# in debug mode
    &log ("Error: $msg \n");
}

	
sub displaySyntax {
# Displays a help message

	my ( $Script ) = ( $0 =~ m#([^\\/]+)$#);
	my $Line = "-" x length ( $Script );
	print << "EOT";
	
$Script
$Line
Spong Network Monitor -- Windows NT Client.
Version: $VERSION

Syntax: 
perl $Script [--remove][--install][--debug][--nosleep][--help]  
             [--start][--stop][--restart]
			 
    --install........Installs this script as a Win32 service.
    --remove.........Removes this script as a Win32 service.
    --debug..........Runs Spong NT Client in debug mode
                     (in a console window not as a service)
    --start..........Starts Spong NT Client as a Win32 service
    --stop...........Stops the Spong NT Client service
    --restart........Restarts the Spong NT Client service
    --help...........Displays this help message				 
    --nosleep........Runs the checks once, then exits.
                     Use in conjunction with --debug.
Examples:
    perl $Script --install
    perl $Script --start
    perl $Script --remove
    perl $Script --debug
    perl $Script --debug --nosleep

IMPORTANT: After installing Spong NT Client, you must start it with
           perl $Script --start, or 
           Control Panel-->Services-->Spong-NT-Client-->Start,
           Running the script from command line with no parameters will
           not start it properly.
EOT

}

